function [X Xsol] = OneExpThree(affinityMatrix,group1,group2)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%  ni    : number of inliers
%%%%%%%% npo    : number of outliers in the first point set
%%%%%%%% nqo    : number of outliers in the second point set
%%%%%%%% sigma  : deformation parameter
%%%%%%%% sigmad : sensitivity parameter
%%%%%%%% K2    : the number of selected largest modes
%%%%%%%% K3    : the number of nearest neighbors for a mode
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


ni = size(group1,2);
np = size(group1,2);
nq = size(group2,2);
n = np*nq;
K2=100;
K3=min(1000,np*nq);

%%Affinity Matrix
M1 = affinityMatrix;
[col, row, val] = find(triu(affinityMatrix)');
M2 = [row, col, val];
NumOfEntry1 = size(M2,1);
Index = find(M2(:,3)>2.25);
NumOfEntry2 = size(Index,1);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%Spectral Method%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%First Eigvector,make sure its sign is correct
tic;
opts.disp = 0;
[V,D]=eigs(M1,1,'LM',opts);
ESum = sum(V);
if ESum<0
    V=-V;   
end;
T = toc;

%%Recover correspondences
[Y,I] = sort(V,'descend');
Index = find(Y>1e-10); %%only deal with nozero elements
CurNum = size(Index,1);
CorsSM = ConstructCorrespondence(I(1:CurNum),CurNum,np,nq);
% CorrectCor = intersect(GroundCors,Cors);
% Ncc = size(CorrectCor,1);
TimeUsed = T;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%SEA%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%evolve from every vertices
StartPoints = [];
for i=1:n
    StartPoints{i} = [i,1];
end;
StartPoints = StartPoints';

K1 = ni;
Neighbor = [];
Para = [];
Para(1) = n;
Para(2) = K1;
Para(3) = 0.1/ni;
Para(4) = 0.00001;
Para(5) = 0.0001;
Para(6) = 0.000000000000001;
tic;
[Maxima,Support,FValue] = SEA(M2,Neighbor,StartPoints,Para');
T = toc;

%%Select K2 largest KKT points
[Y,I] = sort(FValue,'descend');
SelectSupport = [];
SelectMaxima  = [];
for i=1:K2
    SelectSupport{i} = Support{I(i)};
    SelectMaxima{i}  = Maxima{I(i)};
end;
SelectSupport = SelectSupport';
SelectMaxima  = SelectMaxima';

Para(1) = n;
DSs = RecoverDenseSubgraph(M2,SelectSupport,SelectMaxima,Para);

%%Find knn of these dense subgraphs
Para = [];
Para(1) = n;
Para(2) = K3;
NN = KNN(M2,DSs,Para);

%%Recover correspondence
CorrectCorArray = [];
solutions=zeros(K2,n);
for i=1:K2   
%for i=1:1   
    I = NN(:,i);
    Cors = ConstructCorrespondence(I,K3,np,nq);
%     CorrectCor = intersect(GroundCors,Cors);
%     NumOfCorrectCors = size(CorrectCor,1);
%     CorrectCorArray = [CorrectCorArray;NumOfCorrectCors];
    X = zeros(n,1);
    X(Cors) = 1;
    solutions(i, :)=X';
end;

solutions=unique(solutions, 'rows')
uniqueSol=size(solutions,1);
solutionsScore=zeros(uniqueSol, 1);
for k=1:uniqueSol
    solutionsScore(k)=solutions(k,:)*affinityMatrix*solutions(k,:)';
end

[topScore topScoreIndex]=sort(solutionsScore, 'descend')
solutions=solutions(topScoreIndex, :);


% Xsol=zeros(nodesNum,2);
% Xsol(topSolutions(1,:,1),1)=1; %%%v%%%
% if sum(topSolutions(2,:,1))~=0
%     Xsol(topSolutions(2,:,1),2)=1; %%%v%%%
% end

Xsol=solutions(1:2,:)'

% Ncc =[Ncc,max(CorrectCorArray)];
TimeUsed = [TimeUsed,T];



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%Replicator Equation%%%%%%%%%%%%%%%%%%%%%%
% Para = [];
% Para(1) = n;
% Para(2) = 0.00001;
% tic;
% [Maxima,FValue] = ReplicatorEquation(M2,Para);
% T = toc;
% 
% %%Find knn of this KKT point
% SelectedSupport = [];
% SelectedMaxima  = [];
% Sp = 1:n;
% SelectedSupport{1} = Sp';
% SelectedMaxima{1}  = Maxima;
% 
% Para(1) = n;
% DSs = RecoverDenseSubgraph(M2,SelectedSupport,SelectedMaxima,Para);
% 
% Para = [];
% Para(1) = n;
% Para(2) = K3;
% NN = KNN(M2,DSs,Para);
% 
% %Recover correspondences
% Cors = ConstructCorrespondence(NN(:,1),K3,np,nq);
% X = zeros(n,1);
% X(Cors) = 1;

% CorrectCor = intersect(GroundCors,Cors);
% Ncc = [Ncc, size(CorrectCor,1)];
% TimeUsed = [TimeUsed,T];
