%% Makes current problem into Graph Matching form
function [X Xraw] = TBMA(affinityMatrix, group1, group2) 


aff=affinityMatrix;
%how many multiple graphs we have 
m=length(aff)/length(group1);
graphSize=length(group1); 


%parameters

penalty=9;  %PARAM
q=20;        %number of solutions/local optima %PARAM



%variable initialization

nodesNum=length(aff);       %number of nodes in the association graph = n^2
X=zeros(nodesNum,1);        %output to fill in with the final matching X in 0/1 format
Xraw=X;                     %assigned not make an error
kNodes=sqrt(nodesNum);      %number of nodes in the subgraph K (kNodes=sum of 1s in X), also refered here as n=kNodes %%v%%
solutions=zeros(q,kNodes,2);%number of solutions X    
solutionsSum=zeros(q,1);    %the objective score of solution X
noUpdates=0;                
maxAllScore=0;              %among all the solutions X seen so far, maximum sum of affinities alpha of all the nodes in K
maxAllPermO=zeros(1,nodesNum); %among all the solutions X seen so far, vector where the first kNodes entries are the subgraph producing maximum score in K




% penalize association graph
for i=1:m
    i1=graphSize*(i-1); i2=graphSize*i; j1=graphSize*(i-1); j2=graphSize*i;
    aff(i1+1:i2,j1+1:j2) = aff(i1+1:i2,j1+1:j2).*~full(getConflictMatrix(group1, group2));
    aff(i1+1:i2,j1+1:j2) = aff(i1+1:i2,j1+1:j2)-penalty*full(getConflictMatrix(group1, group2)); %KEY nxn
end


for iq=1:q

%(1) random
permO=randperm(kNodes); %KEY 

%(2) random constrained
% permO=zeros(1,k);
% for io=1:nodesNum/k
%     permO(io)=k*(io-1)+ceil(k*rand());
% end;

%random permutation to fill randomly the nodes in the clique to those in
%the intersection
%and to fill in the rest of the (e.g. 380) spaces
 randPerm=randperm(nodesNum);
 Z=setxor(randPerm, permO); %I
 permO=[permO(1:kNodes) Z]; %I

%output: 
%maxPermO - vector where the first kNodes entries are the subgraph K
%maxPermContr - affinities alpha between a node and the subgraph K
%maxWeightsSum - sum of affinities (alpha) of all the nodes in subgraph K
%tic
[maxPermO maxPermContr maxWeightsSum]=tabu3(aff, permO, kNodes);
%toc
if maxWeightsSum>maxAllScore
    maxAllScore=maxWeightsSum; %VI
    maxAllPermO=maxPermO;
end

%recording all the solutions X:
solutions(iq,:, :)=maxPermContr;
solutionsSum(iq)=sum(solutions(iq,:, 2))/2; %should be the same as maxWeightsSum

end %for q


%at this point, all the compuations are done, we just work with the result


%printing solutions elegantly in a file

fileId=fopen('solutions.txt','at');
ssol=sort(solutions, 2); %VIMP
fprintf(fileId, '\n After tabu \n');
fprintf(fileId, '%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\n', ssol(:,:,1));
%fclose(fileId);


%ours

%see the maximum subgraph, sorted for better visualization
sort(maxAllPermO(1:kNodes)); %VIMP

%lists the matches between a node in G1 and a node in G2
visOurMatch=[ceil(maxAllPermO(1:kNodes)/20); subs(mod(maxAllPermO(1:kNodes),20), 0, 20)]';%I
%makes the list of matches into a 0-1 vector
X=zeros(nodesNum,1);
X(maxAllPermO(1:kNodes))=1;



%ground truth

%for visualization, list the ground truth matches
%gr=find(cdata.GTbool(:)>0.5)';
%visGroundMatch=[ceil(gr/20); subs(mod(gr,20), 0, 20)] %I

%ground truth score (counterpart of ours maxWeightsSum)
%groundScore=cdata.GTbool(:)'*cdata.affinityMatrix*cdata.GTbool(:) %I



end