function [maxPermO maxPermAff maxWeightsSum]=tabu(aff, permO, k)

%inputs
%aff - n^2xn^2 affinity matrix
%permO - permuattion of nodes, first k nodes indicate the nodes in the subgraph K
%k - indicates the number of nodes in the subgraph

%%% algorithm
cls=5; %PARAM
aTabu=2; %PARAM
bTabu=4; %PARAM
alpha=1500; %PARAM

nodesNum=length(permO);
alphaO=zeros(nodesNum, 1);      %affinity of each node to the clique
moveGains = zeros(cls);         %delta, gains from switching two nodes from K and L
tabuIters=zeros(nodesNum,1);    %record tabu iterations for every node
iter=0;
nonImpIter=0;

bestTabu=-100000*ones(3,1);     %first row - delta change in objective, second/third - nodes we exchange
bestNonTabu=-100000*ones(3,1);  %-||-
maxPermO=permO;                 %all nodes in the graph
maxPermAff=zeros(k,2);        %only nodes in the clique, first row - nodes numbers, second - affinity to the rest of the clique

%% compute initial affinities

%for each node calculate the sum of affinity values to those inside
% the clique (we don't need outside, it's not our objective)
for i=1:nodesNum
        alphaO(i)=sum(aff(i,permO(1:k)));
end;

%find the sum of the initial clique and make it maxsum for initialization
maxWeightsSum=sum(alphaO(permO(1:k)))/2; %the largest objective found so far
weightsSum=sum(alphaO(permO(1:k)))/2; %the objective of the curreny subgraph


%% keep exchanging nodes

while (nonImpIter<alpha)
%for iuu=1:10000    
if i~=-100000
    
    %step 2
    %for each node in and out of clique compute the node affinity value to the clique, 
    %that is sum values to the nodes in clique (first row) and add indexes of each nodes 
    %(second row of inClique & outClique)

    inClique= [alphaO(permO(1:k))'; permO(1:k)];
    outClique= [alphaO(permO(k+1:end))'; permO(k+1:end)];

    %sort nodes inside the clique to find one with least affinity
    [sorted, ind]=sort(inClique(1,:));
    sortedInClique=inClique(:, ind);
    
    %sort nodes outside the clique to find one with most affinity to clique
    [sortedOut, indOut]=sort(outClique(1,:), 'descend'); 
    sortedOutClique=outClique(:, indOut);

    %step 3
    %compute move gains for top cls nodes from sortedInClique and sortedOutClique
    for i=1:cls
        for j=1:cls
        moveGains(i,j)=-sortedInClique(1,i)+sortedOutClique(1,j)-aff(sortedInClique(2,i), sortedOutClique(2,j));
        end
    end

    %%%%%%%%%%%%%%%%%%%%%%%%%3

    %tic

    %see which move is the best, method 1
    %     deltaTop=zeros(cls*cls,3);
    %     for i=1:cls
    %         for j=1:cls
    %             deltaTop((i-1)*cls+j, 1)=moveGains(i,j);
    %             deltaTop((i-1)*cls+j, 2)=i;
    %             deltaTop((i-1)*cls+j, 3)=j;
    %         end;
    %     end
    % 
    %     [aa bb]=sort(deltaTop(:,1), 'descend');
    %     sortedDeltaTop=deltaTop(bb,:);
    %     %end information
    % 
    %
    %
    %   moveDone=false; c=0;
    %     while moveDone==false & c<cls*cls
    % 
    %     c=c+1;
    % 
    %     ix=sortedDeltaTop(c, 2);
    %     jx=sortedDeltaTop(c, 3);
    %     nodeix=sortedInClique(2,ix);
    %     nodejx=sortedOutClique(2,jx);
    % 
    %     if weightsSum+sortedDeltaTop(c)>maxWeightsSum
    %         best2=sortedDeltaTop(c, 1);
    %         i2=nodeix;
    %         j2=nodejx;
    %         moveDone=true;
    %     elseif tabuIters(nodeix)<=iter & tabuIters(nodejx)<=iter 
    %          best2=sortedDeltaTop(c, 1);
    %          i2=nodeix;
    %          j2=nodejx;
    %          moveDone=true;
    %     end
    % 
    %     end
    % 
    %  if moveDone==false
    %      best2=0;
    %      i2=1;
    %      j2=1;
    %      moveDone=true;
    %  end
    %  
    %  
    %  i=i2; j=j2; best=best2;
    %t3=toc
    %

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

 %tic
    %here we find the best current and update it
    %we take the tabu only if it increases the max objective
    %otherwise we take the best nontabu and update the current and the
    %weights
    %step next
    for iii=1:cls
        for jjj=1:cls

        %i and j are node indices of top nodes from cls
        %iii and jjj are just numbers from 1 to cls
         i=sortedInClique(2,iii);
         j=sortedOutClique(2,jjj);

         if (tabuIters(i)>iter | tabuIters(j)>iter) & moveGains(iii,jjj)>bestTabu(1)
             bestTabu(1)=moveGains(iii,jjj);
             bestTabu(2)=i;
             bestTabu(3)=j;
         elseif tabuIters(i)<=iter & tabuIters(j)<=iter & moveGains(iii,jjj)>bestNonTabu(1)
             bestNonTabu(1)=moveGains(iii,jjj);   
             bestNonTabu(2)=i;
             bestNonTabu(3)=j;
         end

         if  (bestTabu(1) > bestNonTabu(1)) & (weightsSum+bestTabu(1)>maxWeightsSum)
             best=bestTabu(1);
             i=bestTabu(2);
             j=bestTabu(3);
         else    % can lower the objective
             best=bestNonTabu(1);
             i=bestNonTabu(2);
             j=bestNonTabu(3);
         end

        end;  
    end;
 %t4=toc;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%4

    if i==-100000
        break;
    end

    %% (information) 
    %test of two methods are equivalent
    
    % if i2~=i | j2~=j | best ~=best2
    %     best
    %     i 
    %     tabuIters(i)>iter
    %     j
    %     tabuIters(j)>iter
    %     best2
    %     i2
    %     tabuIters(i2)>iter
    %     j2 
    %     tabuIters(j2)>iter
    %     excep=1;
    % 
    % end
    %
    % (information) 
    %shows that tabu lists shrinks and expands
    
    aq = find(tabuIters>iter); %I
    [aq tabuIters(aq)]; %I
    [i j]; %I
    best; %I
    
    %% line 12 algo 2
     weightsSum=weightsSum+best; %I

    %% switch a node in L with a node in clique K
    ii=find(permO==i);
    jj=find(permO==j);
    temp=permO(ii);
    permO(ii)=permO(jj);
    permO(jj)=temp;

    %make the switched variables tabu
    tabuIters(i)=ceil(aTabu+bTabu*rand())+iter;
    tabuIters(j)=ceil(aTabu+bTabu*rand())+iter;

    %% update the weights

    %inClique
    alphaO_ = alphaO;
    b = false(size(alphaO));
    b(1:k) = true;
    alphaO_(b) = alphaO_(b)-aff(i,b)'+aff(j,b)';
    if 1<=i && i<=k
        alphaO_(i) = alphaO(i)+aff(i,j);
    end
    alphaO = alphaO_;
    
    
%     for ii=1:k % k is the length of the nodes desired in clique
%         if ii==i
%             alphaO(ii)=alphaO(i)+aff(i,j); %if in both U (and Z) are positive %corrected version
%         else
%             alphaO(ii)=alphaO(ii)-aff(i,ii)+aff(j,ii); %if in both U (and Z) are positive
%         end
%     end;

    alphaO_ = alphaO;
    b = false(size(alphaO));
    b(k+1:length(permO)) = true;
    alphaO_(b) = alphaO_(b)-aff(i,b)'+aff(j,b)';
    if k+1<=j && j<=length(permO)
        alphaO_(j) = alphaO(j)-aff(i,j);
    end
    alphaO = alphaO_;

%     %outClique
%     for ii=k+1:length(permO)
%          if ii==j
%             alphaO(ii)=alphaO(j)-aff(i,j); %if in both U (and Z) are positive
%         else
%             alphaO(ii)=alphaO(ii)-aff(i,ii)+aff(j,ii); 
%          end
%     end;

       
    %sum we want to minimize, our f(x)
    %should be equal all three
    sumAlphas=sum(alphaO(permO(1:k)))/2;
    weightsSum;
    
    %% after we switched
     
    if weightsSum>maxWeightsSum
        maxWeightsSum=weightsSum; %I
        maxPermO=permO;
        nonImpIter=0;

        tobesortedpermO=permO(1:k)';
        maxPermAff= [permO(1:k)'  alphaO(permO(1:k))]; 
        [a b]=sort(alphaO(permO(1:k)), 'descend');      %sort nodes in the clique according to their affinities to the rest of the clique
        maxPermAff=[tobesortedpermO(b) a];

    else %if we haven't improved the current solution (maxWeightsSum), although we did move, but the current solution maybe lower than the local optimum
        nonImpIter=nonImpIter+1; %I
    end

    iter=iter+1;  
    bestTabu=-100000*ones(3,1);
    bestNonTabu=-100000*ones(3,1);
end
%iuu=iuu+1;
end; %end while iter

end

