%% Methods & Settings
% Script for setting algorithms to run
%
% Minsu Cho, Jungmin Lee, and Kyoung Mu Lee, 
% Reweighted Random Walks for Graph Matching, 
% Proc. European Conference on Computer Vision (ECCV), 2010
% http://cv.snu.ac.kr/research/~RRWM/
% Updated at Nov 1st, 2011

% You can add an algorithm following the script below
%nMethods = 1;
%methods(nMethods).fhandle = @fhandle;                         % Function of the algorithm
%methods(nMethods).variable = {'var1', 'var2', 'var3'};        % Input variables that the algorithm requires
%methods(nMethods).param = {'name1', 'val1', 'name2', 'val2'}; % Default parameter values
%methods(nMethods).strName = 'algorithm name';                 % Algorithm name tag
%methods(nMethods).color = 'color';                            % Color for plots
%methods(nMethods).lineStyle = 'line style';                   % Line style for plots
%methods(nMethods).marker = 'marker';                          % Marker for plots

nMethods = 0;

%% oo
 if 0
     nMethods = nMethods + 1;
     methods(nMethods).fhandle = @HADGA;
     methods(nMethods).variable = {'affinityMatrix', 'E12'};
     methods(nMethods).param = {};
     methods(nMethods).strName = 'HADGA';
     methods(nMethods).color = 'm';
     methods(nMethods).lineStyle = '-';
     methods(nMethods).marker = '*';
 end

%% oo
 if 0
     nMethods = nMethods + 1;
     methods(nMethods).fhandle = @TBMA;
     methods(nMethods).variable = {'affinityMatrix', 'group1', 'group2'};
     methods(nMethods).param = {};
     methods(nMethods).strName = 'TBMA';
     methods(nMethods).color = 'b';
     methods(nMethods).lineStyle = '-';
     methods(nMethods).marker = '*';
 end
 
 %% oo
 if 0
     nMethods = nMethods + 1;
     methods(nMethods).fhandle = @TBMAyumin;
     methods(nMethods).variable = {'affinityMatrix', 'group1', 'group2'};
     methods(nMethods).param = {};
     methods(nMethods).strName = 'TBMAyumin';
     methods(nMethods).color = 'g';
     methods(nMethods).lineStyle = '-';
     methods(nMethods).marker = '*';
 end
 
 %%
 if 1
     nMethods = nMethods + 1;
     methods(nMethods).fhandle = @TBMA_reproduce_yumin;
     methods(nMethods).variable = {'affinityMatrix', 'group1', 'group2'};
     methods(nMethods).param = {};
     methods(nMethods).strName = sprintf('Proposed \n (Short-term) cleaned'); %Penlaty
     methods(nMethods).color = [1 0 1];
     methods(nMethods).lineStyle = '-';
     methods(nMethods).marker = 'o';
 end
     %% oo
 if 1
     nMethods = nMethods + 1;
     methods(nMethods).fhandle = @TBMA3test2;
     methods(nMethods).variable = {'affinityMatrix', 'group1', 'group2'};
     methods(nMethods).param = {};
     methods(nMethods).strName = sprintf('Proposed \n (Short-term)'); %Penlaty
     methods(nMethods).color = [ 0.9047    0.1918    0.1988];
     methods(nMethods).lineStyle = '-';
     methods(nMethods).marker = 'o';
 end
 
 
  %% oo
 if 0
     nMethods = nMethods + 1;
     methods(nMethods).fhandle = @TBMA3;
     methods(nMethods).variable = {'affinityMatrix', 'group1', 'group2'};
     methods(nMethods).param = {};
     methods(nMethods).strName = sprintf('Proposed \n (Long-term)');
     methods(nMethods).color = [ 0.9047    0.1918    0.1988];
     methods(nMethods).lineStyle = '-';
     methods(nMethods).marker = '*';
 end
 
   %% oo
  if 0
     nMethods = nMethods + 1;
     methods(nMethods).fhandle = @TBMA3test;
     methods(nMethods).variable = {'affinityMatrix', 'group1', 'group2'};
     methods(nMethods).param = {};
     methods(nMethods).strName = sprintf('No penalty');
     methods(nMethods).color = 'b';
     methods(nMethods).lineStyle = '-';
     methods(nMethods).marker = 'o';
 end  
   
   
 if 0
     nMethods = nMethods + 1;
     methods(nMethods).fhandle = @TBMA3;
     methods(nMethods).variable = {'affinityMatrix', 'group1', 'group2'};
     methods(nMethods).param = {};
     methods(nMethods).strName = sprintf('Penalty, 1-to-1');
     methods(nMethods).color = [ 0.9047    0.1918    0.1988];
     methods(nMethods).lineStyle = ':';
     methods(nMethods).marker = '*';
 end
 
 
   %% oo
 if 0
     nMethods = nMethods + 1;
     methods(nMethods).fhandle = @TBMA3test;
     methods(nMethods).variable = {'affinityMatrix', 'group1', 'group2'};
     methods(nMethods).param = {};
     methods(nMethods).strName = sprintf('No penalty, 1-to-1');
     methods(nMethods).color = 'b';
     methods(nMethods).lineStyle = ':';
     methods(nMethods).marker = '*';
 end
 
 %% oo
 if 0
     nMethods = nMethods + 1;
     methods(nMethods).fhandle = @TBMAmulti3;
     methods(nMethods).variable = {'affinityMatrix', 'group1', 'group2'};
     methods(nMethods).param = {};
     methods(nMethods).strName = 'TBMAmulti3';
     methods(nMethods).color = 'c';
     methods(nMethods).lineStyle = '-';
     methods(nMethods).marker = '*';
 end
 
 
 %% FGM
if 0
    nMethods = nMethods + 1;
    methods(nMethods).fhandle = @fgmU;
    methods(nMethods).variable = {'Kp', 'Kq', 'E12', 'gphs', 'asgT','par'};
    methods(nMethods).strName = 'FGM';
    methods(nMethods).param = {};
    methods(nMethods).postDiscretization = 0;
    methods(nMethods).postProcess = @postGreedy;
    methods(nMethods).color = [0.2941    0.5447    0.7494];
    methods(nMethods).lineStyle = '-';
    methods(nMethods).marker = '*';
end
 
 %% RRWM Cho et al. ECCV2010
if 1
    nMethods = nMethods + 1;
    methods(nMethods).fhandle = @RRWM;
    methods(nMethods).variable = {'affinityMatrix', 'group1', 'group2'};
    methods(nMethods).param = {};
    methods(nMethods).strName = 'RRWM';
    methods(nMethods).color =[0.3718    0.7176    0.3612]; 
    methods(nMethods).lineStyle = '-';
    methods(nMethods).marker = '*';
end

%% Graduate Assignment
if 1
    nMethods = nMethods + 1;
    methods(nMethods).fhandle = @GAM;
    methods(nMethods).variable = {'affinityMatrix', 'L12', 'group1', 'group2'};
%     methods(nMethods).variable = {'affinityMatrix', 'L12', 'group1', 'group2', 'par'};
    methods(nMethods).strName = 'GAGM';
    methods(nMethods).color = [0.9550    0.8946    0.4722];
    methods(nMethods).lineStyle = '-';
    methods(nMethods).marker = '*';
    methods(nMethods).postProcess = 'hungarian';
end

%% SEA
if 0
    nMethods = nMethods + 1;
    methods(nMethods).fhandle = @Liu;
    methods(nMethods).variable = {'affinityMatrix','group1','group2'};
    methods(nMethods).strName = 'SEA';
    methods(nMethods).color = [0.6400    0.6400    0.6400];
    methods(nMethods).lineStyle = '-';
    methods(nMethods).marker = '*';
end

 %% IPFP
if 1
    nMethods = nMethods + 1;
    methods(nMethods).fhandle = @ipfp_gm;
    methods(nMethods).variable = {'affinityMatrix', 'L12'};
    methods(nMethods).strName = 'IPFP'; %-author';
    methods(nMethods).color = [1.0000    0.5482    0.1000];
    methods(nMethods).lineStyle = '-';
    methods(nMethods).marker = '*';
    methods(nMethods).postProcess = 'hungarian';
end


%% Spectral Matching Leordeanu et al. ICCV 2005
if 0
    nMethods = nMethods + 1;
    methods(nMethods).fhandle = @SM;
    methods(nMethods).variable = {'affinityMatrix'};
    methods(nMethods).param = {};
    methods(nMethods).strName = 'SM';
    methods(nMethods).color = [0.9718    0.5553    0.7741];
    methods(nMethods).lineStyle = '-';
    methods(nMethods).marker = '*';
end

%% Hyper Graph Matching (by Zasss) 2nd order
if 0
    nMethods = nMethods + 1;
    methods(nMethods).fhandle = @HGM_2nd;
    methods(nMethods).variable = {'affinityMatrix', 'E12'};
    methods(nMethods).strName = 'PM'; %-2nd';
    methods(nMethods).color = [0.6859    0.4035    0.2412];
    methods(nMethods).lineStyle = '-';
    methods(nMethods).marker = '*';
end

%% DDMCMC
if 0
    nMethods = nMethods + 1;
    methods(nMethods).fhandle = @DDMCMC_icpr;
    methods(nMethods).variable = {'affinityMatrix','E12'};
    methods(nMethods).param = {};
    methods(nMethods).postProcess = 'hungarian';
    methods(nMethods).strName = 'DDMCMC';
    methods(nMethods).color = 'g';
    methods(nMethods).lineStyle = '--';
    methods(nMethods).marker = 'x';
end

%% SMCM Suh et al. ECCV 2012
if 0
    nMethods = nMethods + 1;
    methods(nMethods).fhandle = @SMCM;
    methods(nMethods).variable = {'affinityMatrix', 'group1', 'group2'};
    methods(nMethods).param = {'nParticles', 2000, 'tau', 2};
    methods(nMethods).postProcess = 'none';
    methods(nMethods).strName = 'SMCM';
    methods(nMethods).color = 'b';
    methods(nMethods).lineStyle = '-';
    methods(nMethods).marker = 'o';
end


 
%% Show the algorithms to run
disp('* Algorithms to run *');
for k = 1:nMethods, disp([methods(k).strName ' : @' func2str(methods(k).fhandle)]); end; disp(' ')
clear k